<?php
/**
 * Phire CMS
 *
 * LICENSE
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.TXT.
 * It is also available through the world-wide-web at this URL:
 * http://www.phirecms.org/LICENSE.TXT
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@phirecms.org so we can send you a copy immediately.
 *
 * @category   Phire
 * @package    Phire_Form
 * @author     Nick Sagona, III <nick@moc10media.com>
 * @copyright  Copyright (c) 2009-2011 Moc 10 Media, LLC. (http://www.moc10media.com)
 * @license    http://www.phirecms.org/LICENSE.TXT     New BSD License
 */

/**
 * Phire_Form_AddFile
 *
 * @category   Phire
 * @package    Phire_Form
 * @author     Nick Sagona, III <nick@moc10media.com>
 * @copyright  Copyright (c) 2009-2011 Moc 10 Media, LLC. (http://www.moc10media.com)
 * @license    http://www.phirecms.org/LICENSE.TXT     New BSD License
 * @version    1.1.2
 */

class Phire_Form_AddFile extends Moc10_Form
{

    /**
     * Form templates
     * @var array
     */
    protected $_templates = array('main' => null, 'mobile' => null);

    /**
     * $_POST fields
     * @var array
     */
    protected $_postFields = array('plugin' => null,
                                   'site_id' => null,
                                   'section_id' => null,
                                   'file_name_1' => null,
                                   'file_title_1' => null,
                                   'file_desc_1' => null,
                                   'mobile' => null,
                                   'file_id' => 0);

    /**
     * Initialization method for the add file form.
     *
     * @param  array $post
     * @param  string $fle_nme
     * @return void
     */
    public function init($post = null, $fle_nme = null)
    {

        $lang = new Phire_Language();

        $this->_setFields($post, $fle_nme);
        $this->_setTemplates();

        $this->setAttributes('id', 'addFileForm');
        $this->setAttributes('enctype', 'multipart/form-data');

        $sitesAry = Phire_Sites::getAllowedSites(true);

        // Get sections.
        $sects = new Phire_Table_Sections();
        if ($this->_postFields['site_id'] == 0) {
            $sects->findAll('section_id ASC', 'site_id', $sitesAry['first_site_id']);
        } else {
            $sects->findAll('section_id ASC', 'site_id', $this->_postFields['site_id']);
        }
        $sectAry = array('0' => '(' . $lang->__('None') . ')');
        if (isset($sects->rows[0])) {
            foreach ($sects->rows as $sect) {
                $sectAry[$sect->section_id] = $sect->section_title;
            }
        }

        // Create the 'section_id' form element.
        $section_id = new Moc10_Form_Element_Select('section_id', $sectAry, $this->_postFields['section_id']);
        $section_id->setLabel($lang->__('Section:'));

        $brws = new Moc10_Session_Browser();
        $fileClass = ($brws->browser == 'MSIE') ? 'fileInputIE' : 'fileInput';

        // Create the 'file_name_1' form element.
        $file_name_1 = new Moc10_Form_Element('file', 'file_name_1', $this->_postFields['file_name_1']);
        $file_name_1->setLabel($lang->__('File Name:'));
        $file_name_1->setAttributes('size', ((!is_null($this->_postFields['mobile'])) ? '25' : '35'));
        $file_name_1->setAttributes('class', $fileClass);
        $file_name_1->setRequired(true);

        // Create the 'file_title_1' form element.
        $file_title_1 = new Moc10_Form_Element('text', 'file_title_1', $this->_postFields['file_title_1']);
        $file_title_1->setLabel($lang->__('File Title:'));
        $file_title_1->setAttributes('size', ((!is_null($this->_postFields['mobile'])) ? '25' : '35'));

        // Create the 'file_desc_1' form element.
        $file_desc_1 = new Moc10_Form_Element('text', 'file_desc_1', $this->_postFields['file_desc_1']);
        $file_desc_1->setLabel($lang->__('File Description:'));
        $file_desc_1->setAttributes('size', ((!is_null($this->_postFields['mobile'])) ? '25' : '80'));

        // Create the 'submit' form element.
        $submit = new Moc10_Form_Element('submit', 'submit', $lang->__('SAVE'));
        $submit->setAttributes('style', 'float: right;');

        // Set whether it's a mobile form template or not.
        if (!is_null($this->_postFields['mobile'])) {
            $sess = Moc10_Session::getInstance();
            $submit->setAttributes('class', 'submitBtn');
            $site_id = new Moc10_Form_Element('hidden', 'site_id', $sess->cur_site_id);
            $this->setTemplate($this->_templates['mobile']);
        } else {
            $this->setAttributes('onsubmit', "return addFileFrm.fileValidate();");
            $site_id = new Moc10_Form_Element_Select('site_id', $sitesAry['sites'], $this->_postFields['site_id']);
            $site_id->setLabel($lang->__('Add to Site:'));
            $submit->setAttributes('class', 'submitBtn');
            $site_id->setAttributes('onchange', "xmlhttp.getRequest('../core/process/check.section.php?site_id=' + this.value, addFileFrm.changeSection); return false;");
            $this->setTemplate($this->_templates['main']);
        }

        // Add the form element objects to the form.
        $this->addElements(array($site_id, $section_id, $file_name_1, $file_title_1, $file_desc_1, $submit));

    }

    /**
     * Method to set the form templates.
     *
     * @return void
     */
    protected function _setTemplates()
    {

        $lang = new Phire_Language();

        if (!is_null($this->_postFields['plugin'])) {
            $plugin = "<input type=\"hidden\" name=\"plugin\" value=\"{$this->_postFields['plugin']}\" />";
            $plg_lnk = BASE_PATH . CONTENT_PATH . "/extensions/plugins/{$this->_postFields['plugin']}/?sys_path=" . BASE_PATH . SYS_PATH;
            $plg_cancel = "&plugin={$this->_postFields['plugin']}";
        } else {
            $plugin = null;
            $plg_lnk = './files.php';
            $plg_cancel = null;
        }

        $back = $lang->__('Back');
        $blue = $lang->__('Blue fields are required.');
        $donot_wrap = wordwrap($lang->__('Once you click save, DO NOT leave the page until the files have finished processing.'), 80, "<br />");
        $donot = $lang->__('Once you click save, DO NOT leave the page until the files have finished processing.');
        $cancel = $lang->__('CANCEL');

        // Define the main form template.
        $this->_templates['main'] = <<<FORMTEMP
            <table id="headerTable" cellpadding="0" cellspacing="0"><tr><td>&lt;&lt;&lt; <a class="orangeLink" href="{$plg_lnk}">{$back}</a></td></tr></table>
            <table id="dataAltTable" cellspacing="0" cellpadding="0">
                <tr class="grayHeader">
                    <td class="grayHeader">
                        &nbsp;
                    </td>
                </tr>
                <tr>
                    <td style="padding: 15px;">
                        <table id="entryTable" cellspacing="0" cellpadding="0">
                            <tr>
                                <td>
                                    <h4>{$blue}</h4>
                                </td>
                                <td colspan="2" class="textRight">
                                    [{site_id_label}]
                                    [{site_id}]
                                    <span style="padding: 0 10px 0 10px;"> </span>
                                    [{section_id_label}]
                                    <span id="sectionDiv">
                                        [{section_id}]
                                    </span>
                                </td>
                                <td>
                                    &nbsp;
                                </td>
                            </tr>
                            <tr>
                                <td>
                                    [{file_name_1_label}]
                                </td>
                                <td>
                                    [{file_title_1_label}]
                                </td>
                                <td>
                                    [{file_desc_1_label}]
                                </td>
                                <td>
                                    &nbsp;
                                </td>
                            </tr>
                            <tr>
                                <td id="fileNameTD" style="vertical-align: top;">
                                    [{file_name_1}]
                                </td>
                                <td id="fileTitleTD" style="vertical-align: top;">
                                    [{file_title_1}]
                                </td>
                                <td id="fileDescTD" style="vertical-align: top;">
                                    [{file_desc_1}]
                                </td>
                                <td style="vertical-align: top;">
                                    <a href="#" onclick="addFileFrm.addFileInput(); return false;">[+]</a>
                                </td>
                            </tr>
                            <tr>
                                <td colspan="3">
                                    <strong style="float: left; color: #f00;">{$donot_wrap}</strong><a style="margin: 0 0 0 10px; padding-top: 6px; height: 19px; font-size: 0.825em; float: right;" class="submitGrayBtn" href="../core/process/add.files.php?action=cancel{$plg_cancel}">{$cancel}</a> [{submit}] {$plugin}
                                </td>
                                <td>
                                    &nbsp;
                                </td>
                            </tr>
                        </table>
                        <script type="text/javascript">
                            addFileFormInit();
                            var xmlhttp = new JaxXmlHttp();
                        </script>
                    </td>
                </tr>
            </table>
FORMTEMP;

        $mobile = (!is_null($this->_postFields['mobile'])) ? "<input type=\"hidden\" name=\"mobile\" value=\"true\" />" : null;

        // Define the mobile form template.
        $this->_templates['mobile'] = <<<FORMTEMP
            <table id="headerTable" cellpadding="0" cellspacing="0"><tr><td>&lt;&lt;&lt; <a class="orangeLink" href="./files.php">{$back}</a></td></tr></table>
            <table id="dataAltTable" cellspacing="0" cellpadding="0">
                <tr class="grayHeader" style="height; 16px;">
                    <td class="grayHeader" style="height; 16px;">
                        &nbsp;
                    </td>
                </tr>

                <tr>
                    <td style="padding: 5px;">
                        <table id="entryTable" cellspacing="0" cellpadding="0">
                            <tr>
                                <td>
                                    &nbsp;
                                </td>
                                <td>
                                    <h4>{$blue}</h4>
                                </td>
                            </tr>
                            <tr>
                                <td class="textRight">
                                    [{section_id_label}]
                                </td>
                                <td>
                                    [{section_id}]
                                </td>
                            </tr>
                            <tr>
                                <td class="textRight">
                                    [{file_name_1_label}]
                                </td>
                                <td id="fileNameTD" style="vertical-align: top;">
                                    [{file_name_1}]
                                </td>
                            </tr>
                            <tr>
                                <td class="textRight">
                                    [{file_title_1_label}]
                                </td>
                                <td id="fileTitleTD" style="vertical-align: top;">
                                    [{file_title_1}]
                                </td>
                            </tr>
                            <tr>
                                <td class="textRight">
                                    [{file_desc_1_label}]
                                </td>
                                <td id="fileDescTD" style="vertical-align: top;">
                                    [{file_desc_1}]
                                </td>
                            </tr>
                            <tr>
                                <td colspan="2">
                                    <strong style="color: #f00;">{$donot}</strong><br /><br />
                                    <a style="float: right; margin: 0 0 0 5px; padding: 2px 3px 4px 3px; height: 10px; font-size: 0.9em;" class="submitBtnGray" href="../../core/process/add.files.php?action=cancel&mobile=true">{$cancel}</a> [{submit}] {$mobile} [{site_id}]
                                </td>
                            </tr>
                        </table>
                    </td>
                </tr>
            </table>
FORMTEMP;

    }


    /**
     * Method to set the post fields.
     *
     * @param  array $post
     * @param  string $fle_nme
     * @return void
     */
    protected function _setFields($post = null, $fle_nme = null)
    {

        if (!is_null($post)) {
            foreach ($post as $key => $value) {
                if (array_key_exists($key, $this->_postFields)) {
                    $this->_postFields[$key] = (isset($post[$key])) ? $post[$key] : null;
                }
            }
            $this->_postFields['file_name_1'] = $fle_nme;
        }

    }

}
