<?php
/**
 * Phire CMS
 *
 * LICENSE
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.TXT.
 * It is also available through the world-wide-web at this URL:
 * http://www.phirecms.org/LICENSE.TXT
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@phirecms.org so we can send you a copy immediately.
 *
 * @category   Phire
 * @package    Phire_Form
 * @author     Nick Sagona, III <nick@moc10media.com>
 * @copyright  Copyright (c) 2009-2011 Moc 10 Media, LLC. (http://www.moc10media.com)
 * @license    http://www.phirecms.org/LICENSE.TXT     New BSD License
 */

/**
 * Phire_Form_MemberLogin
 *
 * @category   Phire
 * @package    Phire_Form
 * @author     Nick Sagona, III <nick@moc10media.com>
 * @copyright  Copyright (c) 2009-2011 Moc 10 Media, LLC. (http://www.moc10media.com)
 * @license    http://www.phirecms.org/LICENSE.TXT     New BSD License
 * @version    1.1.2
 */

class Phire_Form_MemberLogin extends Moc10_Form
{

    /**
     * Initialization method for the member login form.
     *
     * @param  string $user
     * @param  string $pass
     * @return void
     */
    public function init($user = null, $pass = null)
    {

        $lang = new Phire_Language();

        $this->setAttributes('id', 'memberForm');

        // Create the 'username' form element.
        $username = new Moc10_Form_Element('text', 'username', (string)Moc10_String::setString($user)->escape()->striptags());
        $username->setLabel($lang->__('Username:'));
        $username->setAttributes('size', '35');
        $username->setRequired(true);

        // Create the 'pass' form element.
        $password = new Moc10_Form_Element('password', 'password', (string)Moc10_String::setString($pass)->escape()->striptags());
        $password->setLabel($lang->__('Password:'));
        $password->setAttributes('size', '35');
        $password->setRequired(true);

        $site = new Phire_Table_Sites();
        $site->getSite();

        // If the form has been submitted, verify the username and password.
        if (!is_null($user)) {
            $member_exist = new Phire_Table_Members();
            $member_exist->findBy('username', $user);
            if (!isset($member_exist->member_id)) {
                $username->addValidator('NotEqual', true, $user, $lang->__("That username doesn't exist."));
            } else if ($member_exist->verified == 'No') {
                $username->addValidator('NotEqual', true, $user, $lang->__("That username isn't verified."));
            } else if ($member_exist->approved == 'No') {
                $username->addValidator('NotEqual', true, $user, $lang->__("That username isn't approved."));
            } else if ((!is_null($pass)) && ($site->member_password_encoding == 'On') && (sha1($pass) != $member_exist->password)) {
                $password->addValidator('NotEqual', true, $pass, $lang->__("That password is incorrect."));
            } else if ((!is_null($pass)) && ($site->member_password_encoding == 'Off') && ($pass != $member_exist->password)) {
                $password->addValidator('NotEqual', true, $pass, $lang->__("That password is incorrect."));
            }
        }

        // Set the member register link if allowed.
        $mem_reg = ($site->member_enable_registration == 'No') ? '' : "<a style=\"font-size: 0.85em; font-weight: normal;\" href=\"" . BASE_PATH . "/member/register\">" . $lang->__('Register') . "</a> | ";

        // Create the 'submit' form element.
        $submit = new Moc10_Form_Element('submit', 'submit', $lang->__('LOGIN'));
        $submit->setAttributes('class', 'submitBtn');
        $submit->setAttributes('style', 'float: right;');

        $base_path = BASE_PATH;

        $forgot = $lang->__('Forgot?');

        // Define the form template.
        $tmpl = <<<FORMTEMP
            <table id="memberTable">
                <tr>
                    <td style="text-align: right; vertical-align: top;">
                        [{username_label}]
                    </td>
                    <td>
                        [{username}]
                    </td>
                </tr>
                <tr>
                    <td style="text-align: right; vertical-align: top;">
                        [{password_label}]
                    </td>
                    <td>
                        [{password}]
                    </td>
                </tr>
                <tr>
                    <td>
                        &nbsp;
                    </td>
                    <td>
                        [{submit}]
                        {$mem_reg}<a style="font-size: 0.85em; font-weight: normal;" href="{$base_path}/member/forgot">{$forgot}</a>
                    </td>
                </tr>
            </table>
            <script type="text/javascript">
                document.getElementById('username').focus();
            </script>
FORMTEMP;

        // Set the form template and add the form element objects to the form.
        $this->setTemplate($tmpl);
        $this->addElements(array($username, $password, $submit));

    }

}
