<?php
/**
 * Phire CMS
 *
 * LICENSE
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.TXT.
 * It is also available through the world-wide-web at this URL:
 * http://www.phirecms.org/LICENSE.TXT
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@phirecms.org so we can send you a copy immediately.
 *
 * @category   Phire
 * @package    Phire_Page
 * @author     Nick Sagona, III <nick@moc10media.com>
 * @copyright  Copyright (c) 2009-2011 Moc 10 Media, LLC. (http://www.moc10media.com)
 * @license    http://www.phirecms.org/LICENSE.TXT     New BSD License
 */

/**
 * Phire_Page
 *
 * @category   Phire
 * @package    Phire_Page
 * @author     Nick Sagona, III <nick@moc10media.com>
 * @copyright  Copyright (c) 2009-2011 Moc 10 Media, LLC. (http://www.moc10media.com)
 * @license    http://www.phirecms.org/LICENSE.TXT     New BSD License
 * @version    1.1.2
 */

class Phire_Page
{

    /**
     * URL request
     * @var string
     */
    protected $_url = null;

    /**
     * Preserve flag
     * @var boolean
     */
    protected $_preserve = false;

    /**
     * Template type
     * @var string
     */
    protected $_type = null;

    /**
     * Template
     * @var string
     */
    protected $_tmpl = null;

    /**
     * Page content
     * @var string
     */
    protected $_page = null;

    /**
     * Cache content
     * @var string
     */
    protected $_cache = null;

    /**
     * Page object
     * @var Phire_Table_Pages
     */
    protected $_page_object = null;

    /**
     * Page fields
     * @var array
     */
    protected $_page_fields = array();

    /**
     * Page published
     * @var string
     */
    protected $_published = 'Yes';

    /**
     * Page access
     * @var string
     */
    protected $_page_access = 'Public';

    /**
     * Page section access
     * @var string
     */
    protected $_section_access = 'Public';

    /**
     * Page live
     * @var string
     */
    protected $_page_live = 'Yes';

    /**
     * Site live
     * @var string
     */
    protected $_site_live = 'Yes';

    /**
     * Use SSL connection
     * @var string
     */
    protected $_use_ssl = 'No';

    /**
     * Site ID
     * @var int
     */
    protected $_site_id = null;

    /**
     * Page 404 flag
     * @var boolean
     */
    protected $_is404 = false;

    /**
     * Page ID
     * @var string|int
     */
    protected $_page_id = null;


    /**
     * Constructor
     *
     * Instantiate the page object.
     *
     * @param  int|string $id
     * @param  int|string $sid
     * @param  boolean $pre
     * @return void
     */
    public function __construct($id, $sid = null, $pre = false)
    {

        $this->_url = $id;
        $this->_preserve = $pre;

        $sess = Moc10_Session::getInstance();
        $lang = new Phire_Language();

        // Handle any mobile flags, if passed.
        if (isset($_GET['mobile'])) {
            switch ($_GET['mobile']) {
                case 'true':
                    $sess->moc10_mobile_bypass = 'FORCE_MOBILE';
                    break;
                case 'false':
                    $sess->moc10_mobile_bypass = 'FORCE_STANDARD';
                    break;
                case 'clear':
                    unset($sess->moc10_mobile_bypass);
                    break;
            }
        }

        // Get site and page data.
        if (!is_null($sid)) {
            $this->_site_id = $sid;
            $site = new Phire_Table_Sites();
            $site->getSite($this->_site_id);
            $this->_setPage($this->_url, $this->_site_id);
        } else {
            // If cache file exists, use the content of the file.
            if (file_exists('.' .  BASE_PATH . CONTENT_PATH . '/cache/' . sha1($this->_url) . '.txt')) {
                $this->_getCache();
            // Else, interact with the database to fetch the page.
            } else {
                $site = new Phire_Table_Sites();
                $site->getSite();
                $this->_site_id = $site->site_id;
                $this->_setPage($this->_url, $this->_site_id);
            }
        }

        // If no cache file found.
        if (is_null($this->_cache)) {

            // If the page isn't found, check for date-based URL or display the 404 error page.
            if ((!isset($this->_page_object->page_id)) || ($this->_page_object->published == 'No') || (($this->_page_object->published == 'Draft') && (!isset($sess->user_id)) && (!isset($_GET['draft']))) || ($this->_page_object->site_live == 'No') || ($this->_page_object->site_id != $this->_site_id) || ($this->_page_object->created > date('Y-m-d H:i:s')) || (($this->_page_object->expire != '0000-00-00 00:00:00') && ($this->_page_object->expire <= date('Y-m-d H:i:s'))) || (($this->_page_object->page_access == 'Private') && (!isset($sess->user_id)) && (!isset($sess->member_id)) && ($this->_page_object->member_enable_login == 'No'))) {

                // Check for a date-based URL.
                $dateAry = $this->_checkDate();

                if (!is_null($dateAry)) {
                    $pages = new Phire_Table_Pages();
                    $memSQL = (!isset($sess->member_id) && !isset($sess->user_id)) ? 'AND page_access != \'Private\' ' : '';
                    if ($dateAry['section_id'] != 0) {
                        $sql = "SELECT * FROM " . DB_PREFIX . "pages WHERE created >= '" . Moc10_String::setString($dateAry['validStartDate'])->escape() . "' AND created <= '" . Moc10_String::setString($dateAry['validEndDate'])->escape() . "' AND section_id = '" . Moc10_String::setString($dateAry['section_id'])->escape() ."' AND published = 'Yes' {$memSQL}ORDER BY created DESC";
                    } else {
                        $sql = "SELECT * FROM " . DB_PREFIX . "pages WHERE created >= '" . Moc10_String::setString($dateAry['validStartDate'])->escape() . "' AND created <= '" . Moc10_String::setString($dateAry['validEndDate'])->escape() . "' AND published = 'Yes' {$memSQL}ORDER BY created DESC";
                    }
                    $pages->execute($sql);

                    if (isset($pages->rows[0])) {
                        $page_content = '';
                        foreach ($pages->rows as $page) {
                            if (($page->expire == '0000-00-00 00:00:00') || ($page->expire > date('Y-m-d H:i:s'))) {
                                if ($page->page_url == $dateAry['realURL']) {
                                    header('Location: ' . BASE_PATH . $page->page_url);
                                }
                                $page_content .= "    <div class=\"archiveDiv\"><a href=\"" . BASE_PATH . $page->page_url . "\">{$page->page_title}</a> (" . date('l, F d, Y', strtotime($page->created)) . ")</div>\n";
                            }
                        }
                        if ($page_content == '') {
                            $page_content = "     <p>" . $lang->__('There is no content during that time period.') . "</p>\n";
                        }
                    } else {
                        $page_content = "     <p>" . $lang->__('There is no content during that time period.') . "</p>\n";
                    }

                    unset($this->_page_object);
                    $this->_type = $site->type;
                    $cfg = new Phire_Table_Config();

                    // Handle mobile presentation if applicable.
                    $mob = new Moc10_Session_Mobile();
                    if ($dateAry['template_id'] > 0) {
                        $tmp = new Phire_Table_Templates();
                        $tmp->findById($dateAry['template_id']);
                    }

                    // Determine template to use.
                    if (($dateAry['template_id'] > 0) && (isset($tmp->template_id))) {
                        $def_tmpl = $tmp->template;
                        $def_mob_tmpl = $tmp->mobile_template;
                    } else if ($dateAry['template_id'] == 0) {
                        $def_tmpl = $cfg->settings['default_system_template'];
                        $def_mob_tmpl = $cfg->settings['default_system_template'];
                    } else if ($dateAry['template_id'] == -1) {
                        $def_tmpl = null;
                        $def_mob_tmpl = null;
                    }

                    // Check if any force mobile flags are set.
                    if (($mob->isMobile !== false) && ($def_mob_tmpl != '')) {
                        $this->_tmpl = (isset($sess->moc10_mobile_bypass) && ($sess->moc10_mobile_bypass == 'FORCE_STANDARD')) ? $def_tmpl : $def_mob_tmpl;
                    } else {
                        $this->_tmpl = (isset($sess->moc10_mobile_bypass) && ($sess->moc10_mobile_bypass == 'FORCE_MOBILE')) ? $def_mob_tmpl : $def_tmpl;
                    }

                    $site_email = (($site->site_email != '') ? $lang->__('Please contact the %1website administrator%2 for assistance.', array("<a href=\"mailto:{$site->site_email}\">", "</a>")) : '');

                    $this->_is404 = false;
                    $this->_page_id = null;

                    $this->_page_fields = array('page_content' => $page_content,
                                                'page_charset' => $site->charset,
                                                'page_url' => $this->_url,
                                                'page_title' => (string)Moc10_String::setString($site->default_title)->strip() . ' &gt; ' . $dateAry['dateTitle'],
                                                'page_sub_title' => $dateAry['dateTitle'],
                                                'page_author' => (string)Moc10_String::setString($site->default_author)->strip()->html(),
                                                'page_keywords' => (string)Moc10_String::setString($site->default_keywords)->strip()->html(),
                                                'page_desc' => (string)Moc10_String::setString($site->default_desc)->strip()->html(),
                                                'page_robots' => $site->robots,
                                                'site_domain' => $site->site_domain,
                                                'site_email' => $site_email,
                                                'base_path' => BASE_PATH,
                                                'sys_path' => SYS_PATH,
                                                'content_path' => CONTENT_PATH);

                // Else, display 404 error page.
                } else {

                    unset($this->_page_object);
                    $this->_type = $site->type;
                    $cfg = new Phire_Table_Config();

                    // Handle mobile presentation if applicable.
                    $mob = new Moc10_Session_Mobile();
                    if ($site->default_template_id > 0) {
                        $tmp = new Phire_Table_Templates();
                        $tmp->findById($site->default_template_id);
                    }

                    // Determine template to use.
                    if (($site->default_template_id > 0) && (isset($tmp->template_id))) {
                        $def_tmpl = $tmp->template;
                        $def_mob_tmpl = $tmp->mobile_template;
                    } else if ($site->default_template_id == 0) {
                        $def_tmpl = $cfg->settings['default_system_template'];
                        $def_mob_tmpl = $cfg->settings['default_system_template'];
                    } else if ($site->default_template_id == -1) {
                        $def_tmpl = null;
                        $def_mob_tmpl = null;
                    }

                    // Check if any force mobile flags are set.
                    if (($mob->isMobile !== false) && ($def_mob_tmpl != '')) {
                        $this->_tmpl = (isset($sess->moc10_mobile_bypass) && ($sess->moc10_mobile_bypass == 'FORCE_STANDARD')) ? $def_tmpl : $def_mob_tmpl;
                    } else {
                        $this->_tmpl = (isset($sess->moc10_mobile_bypass) && ($sess->moc10_mobile_bypass == 'FORCE_MOBILE')) ? $def_mob_tmpl : $def_tmpl;
                    }

                    $this->_is404 = true;
                    $this->_page_id = null;

                    $site_email = (($site->site_email != '') ? $lang->__('Please contact the %1website administrator%2 for assistance.', array("<a href=\"mailto:{$site->site_email}\">", "</a>")) : '');

                    $this->_page_fields = array('page_content' => $site->default_404,
                                                'page_charset' => $site->charset,
                                                'page_url' => $this->_url,
                                                'page_title' => (string)Moc10_String::setString($site->default_title)->strip() . ' &gt; ' . $lang->__('404 Error') . ' &gt; ' . $lang->__('Page Not Found'),
                                                'page_sub_title' => "<span style=\"color: #f00;\">" . $lang->__('404 Error') . " &gt; " . $lang->__('Page Not Found') . "</span>",
                                                'page_author' => (string)Moc10_String::setString($site->default_author)->strip()->html(),
                                                'page_keywords' => (string)Moc10_String::setString($site->default_keywords)->strip()->html(),
                                                'page_desc' => (string)Moc10_String::setString($site->default_desc)->strip()->html(),
                                                'page_robots' => $site->robots,
                                                'site_domain' => $site->site_domain,
                                                'site_email' => $site_email,
                                                'base_path' => BASE_PATH,
                                                'sys_path' => SYS_PATH,
                                                'content_path' => CONTENT_PATH);

                    // Track the 404 request if applicable.
                    if (defined('SITE_TRACKING') && (SITE_TRACKING == 'On') && (!$this->_preserve) && (stripos($this->_url, '/favicon.ico') === false)) {
                        $site404s = new Phire_Table_404s();
                        $site404s->execute("SELECT * FROM " . DB_PREFIX . "404s WHERE page_url = '" . $this->_url . "' AND site_id = '" . $site->site_id . "' LIMIT 1");
                        $sql404 = (isset($site404s->rows[0])) ? "UPDATE " . DB_PREFIX . "404s SET requests = '" . ($site404s->rows[0]->requests + 1) . "' WHERE page_url = '" . $this->_url . "' AND site_id = '" . $site->site_id . "' LIMIT 1" : "INSERT INTO " . DB_PREFIX . "404s SET page_url = '" . $this->_url . "', site_id = '" . $site->site_id . "', requests = '1'";
                        $site404s->execute($sql404);
                    }

                }

            // Else, format and set page content.
            } else {

                $this->_is404 = false;
                $this->_page_id = $this->_page_object->page_id;
                $this->_page_access = $this->_page_object->page_access;
                $this->_use_ssl = (($this->_page_object->force_ssl_connection == 'Yes') || ($this->_page_object->use_ssl_connection == 'Yes')) ? 'Yes' : 'No';
                $this->_page_live = $this->_page_object->published;
                $this->_site_live = $this->_page_object->site_live;

                if ($this->_page_object->section_id != 0) {
                    $sect = new Phire_Table_Sections();
                    $sect->findById($this->_page_object->section_id);
                    $this->_section_access = $sect->section_access;
                    $section_url = $sect->section_url;
                    $section_title = $sect->section_title;
                } else {
                    $section_url = null;
                    $section_title = null;
                }

                if ($this->_page_object->default_title != '') {
                    $title_sep = ($this->_page_object->title_separator != '') ? $this->_page_object->title_separator : ' ' ;
                    $page_title = Moc10_String::setString($this->_page_object->default_title . $title_sep . $this->_page_object->page_title)->strip();
                    $page_sub_title = Moc10_String::setString($this->_page_object->page_title)->strip();
                } else {
                    $page_title = Moc10_String::setString($this->_page_object->page_title)->strip();
                    $page_sub_title = Moc10_String::setString($this->_page_object->page_title)->strip();
                }

                $site_email = (($this->_page_object->site_email != '') ? "<a href=\"mailto:{$this->_page_object->site_email}\">" . $lang->__('site administrator') . "</a>" : '');
                $date_format = ($site->default_time_format != '') ? $site->default_date_format . ' ' . $site->default_time_format : $site->default_date_format;

                $pg_content = $this->filter($this->_page_object->page_content);
                $pg_content = Phire_Comments::init($pg_content, $this->_page_object, $this->_preserve)->process();

                // Else initialize the page properties.
                $this->_page_fields = array('page_content' => $pg_content,
                                            'page_charset' => $this->_page_object->charset,
                                            'page_title' => (string)$page_title,
                                            'page_url' => $this->_url,
                                            'page_sub_title' => (string)$page_sub_title,
                                            'page_author' => (string)Moc10_String::setString($this->_page_object->page_author)->strip()->html(),
                                            'page_keywords' => (string)Moc10_String::setString($this->_page_object->page_keywords)->strip()->html(),
                                            'page_desc' => (string)Moc10_String::setString($this->_page_object->page_desc)->strip()->html(),
                                            'page_robots' => $this->_page_object->robots,
                                            'page_tags' => $this->formatPageTags($this->_page_object->page_tags),
                                            'page_created' => (string)Moc10_String::setString($this->_page_object->created)->date($date_format),
                                            'page_updated' => (string)Moc10_String::setString($this->_page_object->updated)->date($date_format),
                                            'section_url' => $section_url,
                                            'section_title' => $section_title,
                                            'site_domain' => $this->_page_object->site_domain,
                                            'site_email' => $site_email,
                                            'site_tags' => Phire_TagCloud::render($this->_page_object->site_id),
                                            'base_path' => BASE_PATH,
                                            'sys_path' => SYS_PATH,
                                            'content_path' => CONTENT_PATH);

                // Initialize the page template.
                $mob = new Moc10_Session_Mobile();
                $cfg = new Phire_Table_Config();

                if ($this->_page_object->template_id == -1) {
                    $this->_tmpl = null;
                } else if ($this->_page_object->template_id == 0) {
                    if ($this->_page_object->default_template_id == -1) {
                        $this->_tmpl = null;
                    } else if ($this->_page_object->default_template_id == 0) {
                        $this->_tmpl = $cfg->settings['default_system_template'];
                    } else {
                        $tmp = new Phire_Table_Templates();
                        $tmp->findById($this->_page_object->default_template_id);
                        if (isset($tmp->template_id)) {
                            if (($mob->isMobile !== false) && ($this->_page_object->mobile_template != '')) {
                                $this->_tmpl = (isset($sess->moc10_mobile_bypass) && ($sess->moc10_mobile_bypass == 'FORCE_STANDARD')) ? $tmp->template : $tmp->mobile_template;
                            } else {
                                $this->_tmpl = (isset($sess->moc10_mobile_bypass) && ($sess->moc10_mobile_bypass == 'FORCE_MOBILE')) ? $tmp->mobile_template : $tmp->template;
                            }
                        }
                    }
                } else {
                    $this->_type = $this->_page_object->type;
                    if (($mob->isMobile !== false) && ($this->_page_object->mobile_template != '')) {
                        $this->_tmpl = (isset($sess->moc10_mobile_bypass) && ($sess->moc10_mobile_bypass == 'FORCE_STANDARD')) ? $this->_page_object->template : $this->_page_object->mobile_template;
                    } else {
                        $this->_tmpl = (isset($sess->moc10_mobile_bypass) && ($sess->moc10_mobile_bypass == 'FORCE_MOBILE')) ? $this->_page_object->mobile_template : $this->_page_object->template;
                    }
                }

                // Track the page request if applicable.
                if (defined('SITE_TRACKING') && (SITE_TRACKING == 'On') && (!$this->_preserve) && ($this->_page_object->published != 'Draft')) {
                    $this->_page_object->execute("UPDATE " . DB_PREFIX . "pages SET page_requests = '" . ($this->_page_object->page_requests + 1) . "' WHERE page_id = '" . $this->_page_object->page_id . "' LIMIT 1");
                }

            }

        }

    }

    /**
     * Set method to set the property to the value of _page_fields[$name].
     *
     * @param  string $name
     * @param  mixed $value
     * @throws Exception
     * @return void
     */
    public function __set($name, $value)
    {

        // Check to see if the field key exists.
        if (!array_key_exists($name, $this->_page_fields)) {
            throw new Exception("The field '" . $name . "' does not exist.");
        } else {
            $this->_page_fields[$name] = $value;
        }

    }

    /**
     * Get method to return the value of _page_fields[$name].
     *
     * @param  string $name
     * @throws Exception
     * @return mixed
     */
    public function __get($name)
    {

        // Check to see if the field key exists.
        if (!array_key_exists($name, $this->_page_fields)) {
            throw new Exception("The field '" . $name . "' does not exist.");
        } else {
            return $this->_page_fields[$name];
        }

    }

    /**
     * Output the page content as is.
     *
     * @return string
     */
    public function __toString()
    {

        return $this->_page;

    }

    /**
     * Filter page content.
     *
     * @param  string $content
     * @return string
     */
    public function filter($content)
    {

        // Get page filters and apply them.
        $filterConds = array();
        $filterConds = explode('+', $this->_page_object->page_filter);

        $page_content = '';
        $pgContAry = array();

        // Auto-link any links if the auto-link flag is set.
        if (isset($filterConds[1]) && isset($filterConds[2])) {
            $page_content = (string)Moc10_String::setString($content)->links(true);
        } else if (isset($filterConds[1])) {
            $page_content = (string)Moc10_String::setString($content)->links();
        } else {
            $page_content = $content;
        }

        switch ($filterConds[0]) {
            case '<br />':
                $page_content = (string)Moc10_String::setString($page_content)->br();
                break;

            case '<p>':
                $pgCont = '';
                $pgContAry = explode("\n", $page_content);
                foreach ($pgContAry as $value) {
                    $pgCont .= "<p>" . Moc10_String::setString($value)->trim() . "</p>";
                }
                $pgCont = str_replace("<p></p>", "", $pgCont);
                $pgCont = str_replace("</p><p>", "</p>\n\n<p>", (string)Moc10_String::setString($pgCont)->trim());
                $page_content = $pgCont;

                break;

        }

        return $page_content;

    }

    /**
     * Format page tags
     *
     * @param  string $tags
     * @return string
     */
    public function formatPageTags($tags)
    {

        $page_tags = '';
        $pgTagAry = array();
        $pgTagOutput = '';

        if ($tags != '') {
            $pgTagAry = explode('|', $tags);
            if (count($pgTagAry) != 0) {
                foreach ($pgTagAry as $value) {
                    $tag = new Phire_Table_Tags();
                    $tag->findById($value);
                    $pgTagOutput .= "    <li><a class=\"tagListLink\" href=\"" . BASE_PATH . "/tag/{$tag->tag}\">{$tag->tag}</a></li>\n";
                }
                $page_tags = "<ul id=\"tagList\">\n" . $pgTagOutput . "</ul>\n";
            } else {
                $page_tags = '';
            }

        } else {
                $page_tags = '';
        }

        return $page_tags;

    }

    /**
     * Set content within the page.
     *
     * @param  string $subject
     * @param  string $replace
     * @return void
     */
    public function setContent($subject, $replace)
    {

        if (!is_null($this->_cache)) {
            $this->_cache = str_replace($subject, $replace, $this->_cache);
        } else {
            $this->_page = str_replace($subject, $replace, $this->_page);
        }

    }

    /**
     * Initialize the page.
     *
     * @return void
     */
    public function init()
    {

        $sess = Moc10_Session::getInstance();

        if (!is_null($this->_cache)) {

            $this->_cache = Phire_Parser::getParser(Phire_Parser_Template::init($this->_cache)->getContent(), $this->_url)->getContent();
            if (strpos($this->_cache, '[{comments}]') !== false) {
                $comment_block = Phire_Comments::init('', $this->_page_object, $this->_preserve)->process();
                $this->_cache = str_replace('[{comments}]', $comment_block, $this->_cache);
            }

        } else {

            $this->_page = (!is_null($this->_tmpl)) ? Phire_Parser_Template::init($this->_tmpl)->getContent() : Phire_Parser_Template::init($this->_page_fields['page_content'])->getContent();

            foreach ($this->_page_fields as $key => $value) {
                $val = ($key == 'page_content') ? Phire_Parser_Template::init($this->_page_fields['page_content'])->getContent() : $value;
                $this->setContent('[{' . $key . '}]', $val);
            }

            // Check for a member registration placeholder.
            if (strpos($this->_page, '[{member_register}]') !== false) {
                if (($this->_page_object->member_allow == 'Yes') && ($this->_page_object->member_enable_registration == 'Yes')) {
                    $mem_reg_form = new Phire_Form_MemberRegister(BASE_PATH . '/member/register', 'post', '        ');
                    $mem_reg_form->init($this->_page_object->member_registration);
                    $this->setContent('[{member_register}]', $mem_reg_form->render(true));
                } else {
                    $this->setContent('[{member_register}]', '');
                }
            }
            // Check for a member login placeholder.
            if (strpos($this->_page, '[{member_login}]') !== false) {
                if (($this->_page_object->member_allow == 'Yes') && ($this->_page_object->member_enable_login == 'Yes')) {
                    $mem_login_form = new Phire_Form_MemberLogin(BASE_PATH . '/member/login', 'post', '        ');
                    $mem_login_form->init();
                    $this->setContent('[{member_login}]', $mem_login_form->render(true));
                } else {
                    $this->setContent('[{member_login}]', '');
                }
            }
            // Check for a member profile placeholder.
            if (strpos($this->_page, '[{member_profile}]') !== false) {
                if (($this->_page_object->member_allow == 'Yes') && (isset($sess->member_id))) {
                    $member = new Phire_Table_Members();
                    $member->findById($sess->member_id);
                    $mem_profile_form = new Phire_Form_MemberProfile(BASE_PATH . '/member/profile', 'post', '        ');
                    $mem_profile_form->init($this->_page_object->member_registration, null, $member->member_id);
                    $this->setContent('[{member_profile}]', $mem_profile_form->render(true));
                } else {
                    $this->setContent('[{member_profile}]', '');
                }
            }
            // Check for a member forgot placeholder.
            if (strpos($this->_page, '[{member_forgot}]') !== false) {
                if ($this->_page_object->member_allow == 'Yes') {
                    $mem_forgot_form = new Phire_Form_MemberForgot(BASE_PATH . '/member/forgot', 'post', '        ');
                    $mem_forgot_form->init();
                    $this->setContent('[{member_forgot}]', $mem_forgot_form->render(true));
                } else {
                    $this->setContent('[{member_forgot}]', '');
                }
            }

            if ($this->_preserve == false) {
                $this->_page = Phire_Parser::getParser($this->_page, $this->_url)->getContent();
            }

        }

    }

    /**
     * Return if page is a 404 error page
     *
     * @return boolean
     */
    public function isError()
    {
        return $this->_is404;
    }

    /**
     * Render the page.
     *
     * @return void
     */
    public function render()
    {

        $sess = Moc10_Session::getInstance();

        // If the page is set to 'Draft', and a user is not logged in, redirect to the home page.
        if (($this->_page_live == 'Draft') && !isset($sess->user_id) && !isset($_GET['draft'])) {

            header('Location: /');

        // If the page is private, and a member/user is not logged in, redirect to the member login page.
        } else if ((($this->_page_access == 'Private') || ($this->_section_access == 'Private')) && ((!isset($sess->member_id)) && (!isset($sess->user_id)))) {

            $sess->request = $this->_url;
            header('Location: ' . BASE_PATH . '/member/login');

        // Else, if SSL is required, re-direct to an SSL connection.
        } else if (($_SERVER['SERVER_PORT'] == 80) && ($this->_use_ssl == 'Yes')) {

            header('Location: https://' . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI']);

        // Else, render the page.
        } else {

            if (!headers_sent()) {
                header('Content-type: ' . $this->_type);
            }
            if ($this->_is404) {
                header("HTTP/1.0 404 Not Found");
            }

            $page_content = (!is_null($this->_cache)) ? $this->_cache : $this->_page;
            $page_content = Phire_Plugins::init($page_content, $this->_url, $this->_site_id, $this->_is404, $this->_page_id)->getContent();

            $page_matches = array();
            preg_match_all('/\[\{page_\d*\}\]/', $page_content, $page_matches);

            if (isset($page_matches[0][0])) {
                foreach ($page_matches[0] as $value) {
                    $str = substr($value, strpos($value, '[{page_') + 7);
                    $id = substr($str, 0, strpos($str, '}'));
                    $placeholder = '[{page_' . $id . '}]';
                    if (isset($this->_page_object) && ($id != $this->_page_object->page_id)) {
                        $pg = new Phire_Table_Pages();
                        $pg->findById($id);
                        $pg_cont = (isset($pg->page_id)) ? $pg->page_content : '';
                    } else if (is_numeric($id)) {
                        $pg = new Phire_Table_Pages();
                        $pg->findById($id);
                        if (isset($pg->page_id)) {
                            $pg_cont = (isset($pg->page_id)) ? $pg->page_content : '';
                        } else {
                            $pg_cont = '';
                        }
                    } else {
                        $pg_cont = '';
                    }
                    $pg_content = str_replace($placeholder, $pg_cont, $page_content);
                }
            } else {
                $pg_content = $page_content;
            }

            if (strpos($pg_content, '[{on}]') !== false) {
                $nav_matches = array();
                preg_match_all('/\[\{on\}\]/', $pg_content, $nav_matches, PREG_OFFSET_CAPTURE);
                if (isset($nav_matches[0][0])) {
                    $swapAry = array();
                    foreach ($nav_matches[0] as $on) {
                        $href = substr($pg_content, ($on[1] + 14));
                        $href = substr($href, 0, strpos($href, '"'));
                        $str = substr($pg_content, $on[1]);
                        $str = substr($str, 0, (strpos($str, '>') + 1));
                        if ($href == substr($_SERVER['REQUEST_URI'], 0, strlen($href)) && (!$this->_is404)) {
                            $swap = str_replace('[{on}]', 'On', $str);
                        } else {
                            $swap = str_replace('[{on}]', '', $str);
                        }
                        $swapAry[] = array($str, $swap);
                    }
                    if (count($swapAry) > 0) {
                        foreach ($swapAry as $swp) {
                            $pg_content = str_replace($swp[0], $swp[1], $pg_content);
                        }
                    }
                }
            }

            print($pg_content);

        }

    }

    /**
     * Get the page fields.
     *
     * @return array
     */
    public function getPageFields()
    {

        return $this->_page_fields;

    }

    /**
     * Get the page type.
     *
     * @return string
     */
    public function getPageType()
    {

        return $this->_type;

    }

    /**
     * Get the page.
     *
     * @return string
     */
    public function getPage()
    {

        return $this->_page;

    }

    /**
     * Get the page.
     *
     * @return string
     */
    public function getMobilePage()
    {

        $this->_tmpl = $this->_page_object->mobile_template;
        $this->init();

        return $this->_page;

    }

    /**
     * Set the page.
     *
     * @param  string $url
     * @param  string|int $sid
     * @return void
     */
    protected function _setPage($url, $sid)
    {

        $page = new Phire_Table_Pages();
        $page->getPage($url, $sid);
        $this->_page_object = $page;

        if (!isset($_GET['query'])) {
            $_GET['query'] = (isset($this->_page_object->page_url)) ? $this->_page_object->page_url : $_SERVER['REQUEST_URI'];
        }

    }

    /**
     * Get the cache file contents.
     *
     * @return void
     */
    protected function _getCache()
    {

        $sess = Moc10_Session::getInstance();
        $mob = new Moc10_Session_Mobile();
        $mobile_folder = null;

        if ($mob->isMobile !== false) {
            $mobile_folder = ($sess->moc10_mobile_bypass == 'FORCE_STANDARD') ? null : 'mobile/';
        } else {
            $mobile_folder = ($sess->moc10_mobile_bypass == 'FORCE_MOBILE') ? 'mobile/' : null;
        }

        // Get cache file.
        $cache_file = new Moc10_File('.' . BASE_PATH . CONTENT_PATH . '/cache/' . $mobile_folder . sha1($this->_url) . '.txt');
        $cache = $cache_file->read();

        // Get cache file headers.
        $cache_file_headers = explode('|', substr($cache, 0, strpos($cache, "\n")));

        // Get page content type, access, published status and SSL settings.
        $this->_type = $cache_file_headers[0];
        $access = explode('_', $cache_file_headers[1]);
        $ssl = explode('_', $cache_file_headers[2]);
        $live = explode('_', $cache_file_headers[3]);
        $this->_site_id = $cache_file_headers[4];
        $this->_page_access = $access[0];
        $this->_section_access = $access[1];
        $this->_use_ssl = (($ssl[0] == '443') || ($ssl[1] == '443')) ? 'Yes' : 'No';
        $this->_page_live = $live[0];
        $this->_site_live = $live[1];

        // Get cached content.
        $this->_cache = substr($cache, (strpos($cache, "\n") + 1));

        // Track page in system if applicable.
        if (defined('SITE_TRACKING') && (SITE_TRACKING == 'On')) {
            if ((!$this->_preserve) && ($this->_page_live == 'Yes') && ($this->_site_live == 'Yes')) {
                $page = new Phire_Table_Pages();
                $page->getPage($this->_url, $this->_site_id);
                $this->_page_object = $page;
                $page->execute("UPDATE " . DB_PREFIX . "pages SET page_requests = '" . ($page->page_requests + 1) . "' WHERE page_id = '" . $page->page_id . "' LIMIT 1");
            } else if ((!$this->_preserve) && (($this->_page_live == 'No') || ($this->_site_live == 'No'))) {
                $site404s = new Phire_Table_404s();
                $site404s->execute("SELECT * FROM " . DB_PREFIX . "404s WHERE page_url = '" . $this->_url . "' AND site_id = '" . $this->_site_id . "' LIMIT 1");
                $sql404 = (isset($site404s->rows[0])) ? "UPDATE " . DB_PREFIX . "404s SET requests = '" . ($site404s->rows[0]->requests + 1) . "' WHERE page_url = '" . $this->_url . "' AND site_id = '" . $this->_site_id . "' LIMIT 1" : "INSERT INTO " . DB_PREFIX . "404s SET page_url = '" . $this->_url . "', site_id = '" . $this->_site_id . "', requests = '1'";
                $site404s->execute($sql404);
            }
        }

    }

    /**
     * Check date-based URL.
     *
     * @return array
     */
    protected function _checkDate()
    {

        $lang = new Phire_Language();
        $months = array(1 => 31, 2 => 28, 3 => 31, 4 => 30, 5 => 31, 6 => 30, 7 => 31, 8 =>31, 9 => 30, 10 => 31, 11 => 30, 12 => 31);
        $dateURL = null;
        $realURL = null;
        $dateTitle = $lang->__('Archive for');
        $sectURL = null;
        $sectID = 0;
        $sectTmpl = 0;
        $validStartDate = null;
        $validEndDate = null;
        $offset = null;

        $regexFullDate = '/^\/((19|20)\d\d\/([1-9]|0[1-9]|1[012])\/([1-9]|0[1-9]|[12][0-9]|3[01])|(19|20)\d\d\/([1-9]|0[1-9]|1[012])\/([1-9]|0[1-9]|[12][0-9]|3[01])\/)$/';
        $regexYearMonth = '/^\/((19|20)\d\d\/([1-9]|0[1-9]|1[012])|(19|20)\d\d\/([1-9]|0[1-9]|1[012])\/)$/';
        $regexYear = '/^\/((19|20)\d\d|(19|20)\d\d\/)$/';

        $matchesFullDate = array();
        $matchesYearMonth = array();
        $matchesYear = array();

        preg_match($regexFullDate, $this->_url, $matchesFullDate, PREG_OFFSET_CAPTURE);
        preg_match($regexYearMonth, $this->_url, $matchesYearMonth, PREG_OFFSET_CAPTURE);
        preg_match($regexYear, $this->_url, $matchesYear, PREG_OFFSET_CAPTURE);

        if (isset($matchesFullDate[0][0])) {
            $dateURL = (substr($matchesFullDate[0][0], -1) == '/') ? substr($matchesFullDate[0][0], 1, -1) : substr($matchesFullDate[0][0], 1);
        } else if (isset($matchesYearMonth[0][0])) {
            $dateURL = (substr($matchesYearMonth[0][0], -1) == '/') ? substr($matchesYearMonth[0][0], 1, -1) : substr($matchesYearMonth[0][0], 1);
        } else if (isset($matchesYear[0][0])) {
            $dateURL = (substr($matchesYear[0][0], -1) == '/') ? substr($matchesYear[0][0], 1, -1) : substr($matchesYear[0][0], 1);
        }

        if (!is_null($dateURL)) {

            $dateAry = explode('/', $dateURL);

            $site = new Phire_Table_Sites();
            $site->getSite();
            $sectTmpl = $site->default_template_id;

            if (isset($dateAry[0]) && (($dateAry[0] % 4) == 0)) {
                $months[2] = 29;
            }

            // On Year, Month and Day.
            if (isset($dateAry[0]) && isset($dateAry[1]) && isset($dateAry[2])) {
                // Month and Day are valid.
                if (($dateAry[1] < 13) && ($dateAry[2] < 32)) {
                    if (($dateAry[2] < 10) && (strlen($dateAry[2]) == 1)) {
                        $day = '0' . $dateAry[2];
                    } else {
                        $day = $dateAry[2];
                    }
                    if (($dateAry[1] < 10) && (strlen($dateAry[1]) == 1)) {
                        $month = '0' . $dateAry[1];
                    } else {
                        $month = $dateAry[1];
                    }
                    if (((int)$dateAry[1] == 2) && ($dateAry[2] > $months[2])) {
                        $day = $months[2];
                    }
                    $validStartDate = $dateAry[0] . '-' . $month . '-' . $day . ' 00:00:00';
                    $validEndDate = $dateAry[0] . '-' . $month . '-' . $day . ' 23:59:59';
                    $dateTitle .= ' ' . date('l, F d, Y', strtotime($validStartDate));
                // Just Month is valid.
                } else if ($dateAry[1] < 13) {
                    if (($dateAry[1] < 10) && (strlen($dateAry[1]) == 1)) {
                        $month = '0' . $dateAry[1];
                    } else {
                        $month = $dateAry[1];
                    }
                    $validStartDate = $dateAry[0] . '-' . $month . '-01 00:00:00';
                    $validEndDate = $dateAry[0] . '-' . $month . '-' . $months[(int)$dateAry[1]] . ' 23:59:59';
                    $dateTitle .= ' ' . date('F Y', strtotime($validStartDate));
                // Just Year is valid.
                } else {
                    $validEndDate = $dateAry[0] . '-01-01 00:00:00';
                    $validEndDate = $dateAry[0] . '-12-31 23:59:59';
                    $dateTitle .= ' ' . date('Y', strtotime($validStartDate));
                }
            // On Year and Month.
            } else if (isset($dateAry[0]) && isset($dateAry[1])) {
                // Just Year and Month is valid.
                if ($dateAry[1] < 13) {
                    if (($dateAry[1] < 10) && (strlen($dateAry[1]) == 1)) {
                        $validStartDate = $dateAry[0] . '-0' . $dateAry[1] . '-01 00:00:00';
                        $validEndDate = $dateAry[0] . '-0' . $dateAry[1] . '-' . $months[(int)$dateAry[1]] . ' 23:59:59';
                    } else {
                        $validStartDate = $dateAry[0] . '-' . $dateAry[1] . '-01 00:00:00';
                        $validEndDate = $dateAry[0] . '-' . $dateAry[1] . '-' . $months[(int)$dateAry[1]] . ' 23:59:59';
                    }
                    $dateTitle .= ' ' . date('F Y', strtotime($validStartDate));
                // Just Year is valid.
                } else {
                    $validStartDate = $dateAry[0] . '-01-01 00:00:00';
                    $validEndDate = $dateAry[0] . '-12-31 23:59:59';
                    $dateTitle .= ' ' . date('Y', strtotime($validStartDate));
                }
            // On Year.
            } else if (isset($dateAry[0])) {
                $validStartDate = $dateAry[0] . '-01-01 00:00:00';
                $validEndDate = $dateAry[0] . '-12-31 23:59:59';
                $dateTitle .= ' ' . date('Y', strtotime($validStartDate));
            }

        }

        if (!is_null($validStartDate) && !is_null($validEndDate)) {
            $retAry = array('validStartDate' => $validStartDate,
                            'validEndDate' => $validEndDate,
                            'section_id' => $sectID,
                            'template_id' => $sectTmpl,
                            'dateTitle' => $dateTitle,
                            'realURL' => $realURL);
        } else {
            $retAry = null;
        }

        return $retAry;

    }

}
