<?php
/**
 * Phire CMS
 *
 * LICENSE
 *
 * This source file is subject to the new BSD license that is bundled
 * with this package in the file LICENSE.TXT.
 * It is also available through the world-wide-web at this URL:
 * http://www.phirecms.org/LICENSE.TXT
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@phirecms.org so we can send you a copy immediately.
 *
 * @category   Phire
 * @package    Phire_Parser
 * @author     Nick Sagona, III <nick@moc10media.com>
 * @copyright  Copyright (c) 2009-2011 Moc 10 Media, LLC. (http://www.moc10media.com)
 * @license    http://www.phirecms.org/LICENSE.TXT     New BSD License
 */

/**
 * Phire_Parser_Feed
 *
 * @category   Phire
 * @package    Phire_Parser
 * @author     Nick Sagona, III <nick@moc10media.com>
 * @copyright  Copyright (c) 2009-2011 Moc 10 Media, LLC. (http://www.moc10media.com)
 * @license    http://www.phirecms.org/LICENSE.TXT     New BSD License
 * @version    1.1.2
 */

class Phire_Parser_Feed
{

    /**
     * Feed matches
     * @var array
     */
    protected $_matches = array();

    /**
     * Feed RegEx
     * @var string
     */
    protected $_regex = '/\[\{feed_\d*\}\]/';

    /**
     * Content to evaluate
     * @var string
     */
    protected $_content = null;

    /**
     * URL request
     * @var string
     */
    protected $_url = null;

    /**
     * Constructor
     *
     * Instantiate the feed parser object.
     *
     * @param  string $content
     * @param  string $url
     * @return void
     */
    public function __construct($content, $url = null)
    {

        // Perform a preg_match_all to look for any feed tags within the content, then parse the content.
        $this->_content = $content;
        preg_match_all($this->_regex, $this->_content, $this->_matches);
        $this->_parse();

    }

    /**
     * Static method to instantiate the feed parser object
     * and return the parsed string.
     *
     * @param  string $cont
     * @param  string $u
     * @return Phire_Parser_Feed
     */
    public static function init($cont, $u = null)
    {

        return new self($cont, $u);

    }

    /**
     * Set content within the content property.
     *
     * @param  string $subject
     * @param  string $replace
     * @return void
     */
    public function setContent($subject, $replace)
    {

        $this->_content = str_replace($subject, $replace, $this->_content);

    }

    /**
     * Get content within the content property.
     *
     * @return string
     */
    public function getContent()
    {

        return $this->_content;

    }

    /**
     * Renders any feed elements within the content.
     *
     * @return void
     */
    protected function _parse()
    {

        // If feed tag elements are found, format and set in content.
        if (isset($this->_matches[0][0])) {

            foreach ($this->_matches[0] as $value) {

                $feed_format = '';

                $fid = str_replace('[{feed_', '', $value);
                $fid = str_replace('}]', '', $fid);

                $feed = new Phire_Table_Feeds();
                $feed->findById($fid);

                $site = new Phire_Table_Sites();
                $site->getSite();

                $date_format = $site->default_date_format;
                $time_format = ($site->default_time_format != '') ? ' ' . $site->default_time_format : '';

                if (isset($feed->feed_id) && ($feed->site_id == $site->site_id)) {

                    try {
                        $limit = ($feed->list_limit > 0) ? $feed->list_limit : null;

                        $fd = new Moc10_Feed_Reader($feed->feed_url, $limit);
                        $fd->setItemTemplate($feed->feed_template);
                        $feed_format .= $fd->render($date_format . $time_format, true);
                        $feed_format = str_replace('[{base_path}]', BASE_PATH, $feed_format);
                        $feed_format = str_replace('[{content_path}]', CONTENT_PATH, $feed_format);
                        $this->setContent($value, $feed_format);

                    } catch (Exception $e) {
                        $this->setContent($value, '');
                    }

                } else {
                    $this->setContent($value, '');
                }

            }

        }

    }

}
