<?php

/**
 * Phire Install Plugins
 *
 * @category   Phire
 * @author     Nick Sagona, III <nick@moc10media.com>
 * @copyright  Copyright (c) 2009-2011 Moc 10 Media, LLC. (http://www.moc10media.com)
 * @license    http://www.phirecms.org/LICENSE.TXT     New BSD License
 * @version    1.1.2
 */

$error = false;
$sort = 'plugin_id';

require_once '../../bootstrap.php';
require_once 'check.plugins.php';

// Install new plugins if detected.
if (count($newPlugins) > 0) {

    // Check if the ZIP extension is installed.
    if (!class_exists('ZipArchive')) {
        header('Location: ' . BASE_PATH . SYS_PATH . '/extensions/plugins.php?error=nozip');
    } else {

        // Check to make sure the files are ZIP files.
        foreach ($newPlugins as $fle) {
            $zipFile = new Moc10_File($fle[1] . $fle[2]);
            if ($zipFile->ext != 'zip') {
                header('Location: ' . BASE_PATH . SYS_PATH . '/extensions/plugins.php?error=file');
            }
        }

        // Iterate through each plugin file.
        foreach ($newPlugins as $fle) {

            $config = new Phire_Table_Config();
            $sysdocroot = $config->settings['system_docroot'];

            // Move the plugin file if necessary.
            $zipFile = new Moc10_File($fle[1] . $fle[2]);
            if (strpos($fle[1], $sysdocroot) === false) {
                if (!file_exists($sysdocroot . BASE_PATH . CONTENT_PATH . '/extensions/plugins/' . $zipFile->basename)) {
                    $zipFile->move($sysdocroot . BASE_PATH . CONTENT_PATH . '/extensions/plugins/' . $zipFile->basename);
                    unset($zipFile);
                    $zipFile = new Moc10_File($sysdocroot . BASE_PATH . CONTENT_PATH . '/extensions/plugins/' . $fle[2]);
                    mkdir($fle[1] . $zipFile->filename);
                }
            }

            // Extract the ZIP file to the correct folder.
            $zip = new ZipArchive();
            if ($zip->open($zipFile->fullpath) === true) {
                $zip->extractTo($sysdocroot . BASE_PATH . CONTENT_PATH . '/extensions/plugins/');
                $zip->close();
            }

            if (file_exists($sysdocroot . BASE_PATH . CONTENT_PATH . '/extensions/plugins/' . $zipFile->filename) && file_exists($sysdocroot . BASE_PATH . CONTENT_PATH . '/extensions/plugins/' . $zipFile->filename . '/' . $zipFile->filename . '.txt')) {

                // Read the plugin config file.
                $pluginTxt = new Moc10_File($sysdocroot . BASE_PATH . CONTENT_PATH . '/extensions/plugins/' . $zipFile->filename . '/' . $zipFile->filename . '.txt');
                $output = $pluginTxt->read();

                // Parse plugin info.
                $plugin = substr($output, (strpos($output, '@plugin') + 7));
                $plugin = trim(substr($plugin, 0, (strpos($plugin, "\n"))));

                $version = substr($output, (strpos($output, '@version') + 8));
                $version = trim(substr($version, 0, (strpos($version, "\n"))));

                $file = substr($output, (strpos($output, '@file') + 5));
                $file = trim(substr($file, 0, (strpos($file, "\n"))));

                $folder = substr($output, (strpos($output, '@folder') + 7));
                $folder = trim(substr($folder, 0, (strpos($folder, "\n"))));

                $author = substr($output, (strpos($output, '@author') + 7));
                $author = trim(substr($author, 0, (strpos($author, "\n"))));

                $desc = substr($output, (strpos($output, '@desc') + 5));
                $desc = trim(substr($desc, 0, (strpos($desc, "*/"))));

                // Parse any plugin tables.
                if (strpos($output, 'TABLES:') !== false) {
                    $tables = substr($output, (strpos($output, 'TABLES:') + 7));
                    if (strpos($tables, "\n") !== false) {
                        $tables = trim(substr($tables, 0, (strpos($tables, "\n"))));
                    } else {
                        $tables = trim($tables);
                    }
                    if ($tables != '') {
                        $tables = str_replace(':', '|', $tables);
                    } else {
                        $tables = '';
                    }
                } else {
                    $tables = '';
                }

                // Parse any plugin subfolders.
                if (strpos($output, 'SUBFOLDERS:') !== false) {
                    $subfolders = substr($output, (strpos($output, 'SUBFOLDERS:') + 11));
                    if (strpos($subfolders, "\n") !== false) {
                        $subfolders = trim(substr($subfolders, 0, (strpos($subfolders, "\n"))));
                    } else {
                        $subfolders = trim($subfolders);
                    }
                    if ($subfolders != '') {
                        $subfolders = str_replace(':', '|', $subfolders);
                        $subAry = explode('|', $subfolders);
                    } else {
                        $subfolders = '';
                    }
                } else {
                    $subfolders = '';
                }

                // Parse any plugin controller files.
                if (strpos($output, 'CONTROLLER:') !== false) {
                    $controller = substr($output, (strpos($output, 'CONTROLLER:') + 11));
                    if (strpos($controller, "\n") !== false) {
                        $controller = trim(substr($controller, 0, (strpos($controller, "\n"))));
                    } else {
                        $controller = trim($controller);
                    }
                } else {
                    $controller = '';
                }

                // Save the new plugin in the DB.
                $pgFields = array('site_id' => $fle[0],
                                  'plugin_name' => $plugin,
                                  'plugin_version' => $version,
                                  'plugin_file' => $file,
                                  'plugin_folder' => $folder,
                                  'plugin_author' => $author,
                                  'plugin_desc' => $desc,
                                  'plugin_tables' => str_replace('[{db_prefix}]', DB_PREFIX, $tables),
                                  'plugin_subfolders' => $subfolders,
                                  'plugin_controller' => $controller);

                $new_plugin = new Phire_Table_Plugins($pgFields);
                $new_plugin->save();

                // Install and execute any necessary database schemas for the plugin.
                if (file_exists($sysdocroot . BASE_PATH . CONTENT_PATH . '/extensions/plugins/' . $folder . '/schema')) {

                    $db = Moc10_Db::getInstance();
                    $schDir = new Moc10_File_Dir($sysdocroot . BASE_PATH . CONTENT_PATH . '/extensions/plugins/' . $folder . '/schema');

                    $schFiles = $schDir->files;
                    sort($schFiles);

                    foreach ($schFiles as $schFile) {
                        $sqlFile = new Moc10_File($sysdocroot . BASE_PATH . CONTENT_PATH . '/extensions/plugins/' . $folder . '/schema/' . $schFile);
                        if (isset($sqlFile->ext) && (strtolower($sqlFile->ext) == 'sql')) {
                            $sql = str_replace('[{db_prefix}]', DB_PREFIX, $sqlFile->read());
                            $db->interface->query($sql);
                        }
                    }

                }

                // Create any necessary subfolders for the plugin.
                if (isset($subAry[0])) {

                    $site = new Phire_Table_Sites();
                    $site->getSite($fle[0]);

                    foreach ($subAry as $sub) {
                        if (!file_exists($site->site_docroot . BASE_PATH . CONTENT_PATH . '/extensions/plugins/' . $folder . '/' . $sub)) {
                            mkdir($site->site_docroot . BASE_PATH . CONTENT_PATH . '/extensions/plugins/' . $folder . '/' . $sub);
                        }
                        chmod($site->site_docroot . BASE_PATH . CONTENT_PATH . '/extensions/plugins/' . $folder . '/' . $sub, 0777);
                    }

                }
            } else {
                $error = true;
            }

            $new_plugin_folder = new Moc10_File_Dir($fle[1] . $folder, true, true);
            $new_plugin_folder->setMode(0777);

            foreach ($new_plugin_folder->files as $file) {
                chmod($file, 0777);
            }


        }

        header('Location: ' . BASE_PATH . SYS_PATH . '/extensions/plugins.php?action=' . (($error) ? 'error' : 'saved'));

    }

} else {
    header('Location: ' . BASE_PATH . SYS_PATH . '/extensions/plugins.php');
}

?>
