<?php

/**
 * Phire Install Index
 *
 * @category   Phire
 * @author     Nick Sagona, III <nick@moc10media.com>
 * @copyright  Copyright (c) 2009-2011 Moc 10 Media, LLC. (http://www.moc10media.com)
 * @license    http://www.phirecms.org/LICENSE.TXT     New BSD License
 * @version    1.1.2
 */

require_once '../bootstrap.php';
require_once '../core/process/install.check.php';
include_once '../core/process/init.php';

if ($sys_check == false) {

    header('Location: ../install.php');

// Else, process the install index page.
} else {

    $installErrors = array();
    $mysqlVersionAry = array();
    $phpVersionAry = array();

    try {

        // Test for system requirements, returning any errors.
        $installErrors = Phire_Dirs::writable($_SERVER['DOCUMENT_ROOT'], true);
        $mysqlVersionAry = explode('-', Moc10_Db::getInstance()->interface->version());
        $phpVersionAry = explode('-', phpversion());

        if (in_array(DB_PREFIX . 'config', Moc10_Db::getInstance()->interface->getTables())) {
            $installErrors[] = "        <p style=\"width: 95%; margin: 0 auto; padding: 5px 0 10px 0;\">" . $lang->__('Phire CMS appears to already be installed on this server. Please check your credentials and try to log in %1here%2.', array("<a href=\"" . BASE_PATH . SYS_PATH . "/\">", "</a>")) . "</p>\n";
        }

        if ($mysqlVersionAry[0] < '5.0') {
            $installErrors[] = "        <p style=\"width: 95%; margin: 0 auto; padding: 5px 0 10px 0;\">" . $lang->__('Phire CMS requires MySQL 5.0 or greater. You currently have %1 installed. Please upgrade MySQL to continue.', $mysqlVersionAry[0]) . "</p>\n";
        }

        if (version_compare(PHP_VERSION, '5.2.6') < 0) {
            $installErrors[] = "        <p style=\"width: 95%; margin: 0 auto; padding: 5px 0 10px 0;\">" . $lang->__('Phire CMS requires PHP 5.2.6 or greater. You currently have %1 installed. Please upgrade PHP to continue.', $phpVersionAry[0]) . "</p>\n";
        }

        // Display install/setup errors if found.
        if (isset($installErrors[0])) {

            include_once '../core/process/init.php';
            include_once '../core/includes/header.php';
            print("        <h2 class=\"redHeader\">" . $lang->__('Install') . " &gt; " . $lang->__('Step 2') . " &gt; " . $lang->__('System Check') . " &gt; " . $lang->__('Failed!') . "</h2>\n");
            foreach ($installErrors as $value) {
                print($value);
            }

            $sys_path = BASE_PATH . SYS_PATH;
            $lng = (isset($_GET['language'])) ? '?language=' . $_GET['language'] : '?language=en';
            $issues = $lang->__('Have you addressed these issues yet?');
            $try = $lang->__('Try Again');

            print <<<INSTALLAGAIN

        <h3 class="blueHeader">{$issues}</h3>
        <p style="width: 95%; margin: 0 auto;"><a style="margin: 0; padding-top: 3px; height: 22px; font-size: 0.825em; width: 12%;" class="submitBtn" href="{$sys_path}/install/{$lng}">{$try}</a></p>

INSTALLAGAIN;
            include_once '../core/includes/footer.php';

        // Else, continue setup.
        } else {

            $lng = (isset($_GET['language'])) ? $_GET['language'] : 'en';
            $sess = Moc10_Session::getInstance();
            $sess->install_check = 'pass1';
            header('Location: ./setup.php?language=' . $lng);

        }

    } catch (Exception $e) {

        include_once '../core/process/init.php';
        include_once '../core/includes/error.header.php';
        print("        <h2 class=\"redHeader\">Install Error</h2>\n");
        print("        <p style=\"width: 95%; margin: 0 auto; padding: 5px 0 10px 0;\">" . $e->getMessage() . "</p>\n");
        include_once '../core/includes/footer.php';

    }

}

?>